package main;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import util.ComplexInfoStatistics;


public class Complex {

	
	public Complex(){
		this(new ArrayList<String>());
	}
	public Complex(List<String> l){
		lst = l;
		isMatch = false;
		precise = 0.0;
		recall = 0.0;
	}
	
	private List<String> lst;
	
	private boolean isMatch;
	private Complex matchedComplex;//¼ƥcomplex
	private double precise;
	private double recall;
	
	private double truePositive;//м  
	
	public void add(String protein){
		lst.add(protein);
	}
	public void updateList(List<String> proteinList){
		lst = proteinList;
	}
	
	public void match(List<Complex> l){
		Complex comp = null;
		double pre = 0.0;
		double rec = 0.0;
		double pvl = -1.0;
		double trueNum = 0;
		for(Complex c : l){
			
			Set<String> set = new HashSet<String>();
			for(String p : lst){
				set.add(p);
			}
			for(String p : c.getProteinList()){
				set.add(p);
			}
			
			double sa = lst.size() + 0.0;
			double sb = c.getProteinList().size() + 0.0;
			double sc = set.size() + 0.0;
			
			double p = (sa+sb-sc) / sa;
			double r = (sa+sb-sc) / sb;
			double pv = getPValue(p, r);
			//System.out.println(sa + "---" + sb + "---" + sc);
			if(pv > pvl){
				pre = p;
				rec = r;
				pvl = pv;
				comp = c;
				trueNum = (sa+sb-sc);
			}
		}
		precise = pre;
		recall = rec;
		truePositive = trueNum;
		matchedComplex = comp;
		isMatch = true;
	}
	public List<Integer> matchThreshold(List<Complex> l, double th){
		List<Integer> rl = new ArrayList<Integer>();
		
		int cur = 0;
		for(Complex c : l){
			
			Set<String> set = new HashSet<String>();
			for(String p : lst){
				set.add(p);
			}
			for(String p : c.getProteinList()){
				set.add(p);
			}
			
			double sa = lst.size() + 0.0;
			double sb = c.getProteinList().size() + 0.0;
			double sc = set.size() + 0.0;
			
			double p = (sa+sb-sc) / sa;
			double r = (sa+sb-sc) / sb;
			
			if(p * r >= th){
				rl.add(cur);
			}
			
			cur++;
		}
		
		return rl;
	}
	public static void matchACC(List<Complex> lcp, List<Complex> lcm){
		
		
		List<Double> lcmOverlapMax = new ArrayList<Double>();
		List<Double> lcpOverlapMax = new ArrayList<Double>();
		for(int i = 0; i < lcp.size(); i++){
			lcpOverlapMax.add(0.0);
		}
		double all = 0.0;
		double ni = 0.0;
		
		for(Complex c : lcm){
			
			ni += c.getProteinList().size();
			
			double max = 0;
			
			for(int i = 0; i < lcp.size(); i++){
			
				Set<String> set = new HashSet<String>();
				for(String p : lcp.get(i).getProteinList()){
					set.add(p);
				}
				for(String p : c.getProteinList()){
					set.add(p);
				}
				
				double sa = lcp.get(i).getProteinList().size() + 0.0;
				double sb = c.getProteinList().size() + 0.0;
				double sc = set.size() + 0.0;
				
				double ol = sa+sb-sc;
				
				all += ol;
				
				if(ol > max){
					max = ol;
				}
				if(ol > lcpOverlapMax.get(i)){
					lcpOverlapMax.set(i, ol);
				}
			}
			
			lcmOverlapMax.add(max);
		}
		System.out.println(lcmOverlapMax.size() + "  " + lcpOverlapMax.size());
		
		double lcmSum = 0.0;
		double lcpSum = 0.0;
		for(Double d : lcmOverlapMax){
			lcmSum += d;
		}
		for(Double d : lcpOverlapMax){
			lcpSum += d;
		}
		
		System.out.println("Sn : " + lcmSum/ni + " PPV : " + lcpSum/all + " Acc : " + Math.sqrt((lcmSum*lcpSum)/(all*ni)));
	}
	public double getPrecise(){
		return precise;
	}
	public double getRecall(){
		return recall;
	}
	public int getPredictNum(){
		return lst.size();
	}
	public int getMatchComplexNum(){
		return matchedComplex.getProteinList().size();
	}
	public double getTruePositiveNum(){
		return truePositive;
	}
	public static double getPValue(double precise, double recall){
		if(precise + recall == 0){
			return 0;
		}
		return (2*precise*recall) / (precise + recall);
	}
	public double getPValue(){
		if(!isMatch){
			return 0.0;
		}
		return getPValue(precise, recall);
	}
	public List<String> getProteinList(){
		return lst;
	}
	
	public Complex getMatchedComplex(){
		return matchedComplex;
	}
	
	
	//723 bou chu li --filter
	public static List<Complex> handleMethod1(List<Complex> lc, Double max_th, Double min_th){
		List<GeneMatch> lgm = testGene.getMatchList(FilePath.filePathMatch);
		
		List<Complex> lcr = new ArrayList<Complex>();
		
		for(Complex c : lc){
			double init = ComplexInfoStatistics.calComplexValue(c, lgm);
			if(init >= max_th){
				lcr.add(c);
			} else if(init >= min_th){
				lcr.add(ComplexInfoStatistics.deleteOneProtein(c, lgm));
			}
		}
		
		return lcr;
	}
	public static List<Complex> handleMethod2(List<Complex> lc, Double max_th, Double min_th){
		List<GeneMatch> lgm = testGene.getMatchList(FilePath.filePathMatch);
		
		List<Complex> lcr = new ArrayList<Complex>();
		
		for(Complex c : lc){
			if(c.getProteinList().size() > 2){
				double init = ComplexInfoStatistics.calComplexValue(c, lgm);
				if(init >= max_th){
					lcr.add(c);
				} else if(init >= min_th){
					lcr.add(ComplexInfoStatistics.deleteOneProtein(c, lgm));
				}
			} else {
				lcr.add(c);
			}
			
		}
		
		return lcr;
	}
	
	//728
	public static void evaluateComplexList(List<Complex> lcp, List<Complex> lcm, double th){
		
		int mn = 0;
		Set<Integer> set = new HashSet<Integer>();
		for(Complex c : lcp){
			List<Integer> li = c.matchThreshold(lcm, th);
			if(li.size() > 0){
				mn++;
				for(Integer i : li){
					set.add(i);
				}
			}
		}
		
		System.out.println("lcp size : " + lcp.size() + "  match num : " + mn);
		System.out.println("lcm size : " + lcm.size() + "  match num : " + set.size());
		System.out.println("pre : " + (mn+0.0)/lcp.size() + "  recall : " + (set.size()+0.0)/lcm.size() + "  f : " + getPValue((mn+0.0)/lcp.size(), (set.size()+0.0)/lcm.size()));
	}
	
}
